/**********************************************************************
 Message Authenticator Algorithm         Beycan KAHRAMAN - 040020337

 usage: program  J  K  file

 Example of Usage:

      deejay@adelante beycan $ nasm -f elf 040020337.asm -o bey1.o
      deejay@adelante beycan $ gcc  -c     040020337.c   -o bey2.o
      deejay@adelante beycan $ gcc  bey1.o bey2.o        -o MAA
      deejay@adelante beycan $ ./MAA 80018001 80018000 data0.txt

      >>     J = 80018001     K = 80018000     f = data0.txt
      >>     Z = DB79FBDC

**********************************************************************/
#include <stdio.h>

#define MAX_ARRAY 30000

/**********************************************************************
Function Name: Z = calculate(J, K, ARRAY, Num)
Description  : This funciton calls other functions in assmly code and
               returns Z, it is written in 040020337.asm.
Variables    : J and K are the input variables of user
               a is the aray of messages that will create Z
               n is the number of messages
**********************************************************************/
unsigned long calculate(unsigned long J,  unsigned long K, 
                        unsigned long *a, unsigned long n);

/**********************************************************************
Function Name: void message(msg)
Description  : prints the error message and exits the program
Variables    : gets the message that will be printed
**********************************************************************/
void message(char *str)
{
  printf("\n%s\n\n", str);
  exit(-1);
}

/**********************************************************************
Function Name: value convert(hexSTRING)
Description  : used to calculate the values of hexSTRINGs
               Ignores all characters that are not hexadecimal
Variables    : gets the hex string and returns the unsigned long 
               value of this string
**********************************************************************/
unsigned long convert(char *num)
{
  unsigned long result = 0;
  int i;

  for(i=0; i<8; i++)
  {
    if(num[i] >= '0' && num[i] <= '9')      result += num[i] - '0';
    else if(num[i] >= 'a' && num[i] <= 'f') result += num[i] - 'a' + 10;
    else if(num[i] >= 'A' && num[i] <= 'F') result += num[i] - 'A' + 10;
    else{
printf("%d %d >> ", num[i], i);
      message("ERROR: undefined character in converting...");
}

    if(i<7)
      result <<= 4;
  }

  return result;
}

/**********************************************************************
Function Name: num readFromFile(file, array)
Description  : reads the messages from file and creates the array with
               num elements
Modules      : We use convert function to calculate the long value of
               hex string. Here we could use
                    fscanf(f, "%lX", &number);
               However, if the file corrupts or there are spaces in
               text file it will not work correctly. Example file:
                  005A 5D34       >>   read 005A5D34
                  56412352        >>   read 56412352
                  654
                    2 354E        >>   read 6542354E
                  ABCDEF 0
                   045 324        >>   read ABCDEF00
                                  >>   ignore  45324

               The second reason why we use convert function is, we 
               need ta convert J and K which the user inputs.
Variables    : gets the file name to be opened
               gets the array, which will be used to get messages
               returns number of messages in the array
**********************************************************************/
unsigned long readFromFile(char *file, unsigned long array[])
{
  FILE *f = 0;
  int i;
  int num = 0;
  char getLine[10] = {0};

  f = fopen(file, "r");
  if( f != 0)
  {
    while(1)
    {
      for(i=0; i<8; ++i)
      {
        fscanf(f, "%c", &getLine[i]);

        if(feof(f))
          return num;
        else if(getLine[i] < '0')
           --i;
      }
      array[num++] = convert(getLine);
    }
    fclose(f);
  }
  else
    message("ERROR: file could not be opened...");
}

/*********************************************************************
     M A I N    F U N C T I O N
**********************************************************************/
int main(int arg, char **argv)
{
    unsigned long array[MAX_ARRAY], numOfElement, J, K;

    if(arg != 4)
        message("usage: program  J  K  file");

    J = convert(argv[1]);
    K = convert(argv[2]);

    numOfElement = readFromFile(argv[3], array);

    printf("\n>>     J = %lX     K = %lX     f = %s\n", J, K, argv[3]);

    printf(">>     Z = %lX\n\n", calculate(J, K, array, numOfElement));

    return 0;
}
